/* 
 * File:   commands.h
 * Author: Manolo Serrano
 * http://tren.enmicasa.net
 *
 * Created on 6 de abril de 2013, 11:42
 *
 *  This file is part of rpic.
 *
 *  rpic is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  rpic is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with rpic.  If not, see <http://www.gnu.org/licenses/>.
 *
 * This file contains the definitions of the commands and type properties of the messages betweern ROCRRAIL adn ROCPI
 * It also includes the ERROR and STATES definitions
 */

#ifndef COMMANDS_H
#define	COMMANDS_H

#ifdef	__cplusplus
extern "C" {
#endif

//Command field within the command/type byte
#define SYSTEMCMD      0
#define ACCNMRACMD     1
#define ACCMMCMD       2
#define LOCMMCMD       3
#define LOCMMXCMD      4
#define LOCNMRACMD     5
#define PROGCMD        6
#define CONFIGCMD      7

//Type of SYSTEMCMD states and errors
#define CYCLEOFF        0       //State and command. POWER ON RAIL
#define CYCLEON        31       //State and command. POWER OFF RAIL
//Other commands to be treated carefully. By default a command is also a state, if this is not the case, it has to be coded on the command triage
#define PTON           20       //Command. Programming track ON
#define PTOFF          30       //Command. Programming track OFF
#define NEWCMD          2       //State, new command received
#define OK              0       //Response: OK
#define ERROR1          1       //State and response: Not to be received: Checksum error    MAIN ERROR
#define ERROR2          2       //State. Not to be sent: defined at server by default
#define ERROR10         10      //Response: Address error.. A number that is different to the normal outputs of get_address functions


//Type field within the command/type byte
//for LOCMMCMD and ACCMMCMD
#define ACCMM                0  //Not implemented so far
#define LOCOMM1              1
#define LOCOMM1R             2  //virtual type to note internally a change in direction. Not to be stored
#define LOCOMM1Fx            3  //Not implemented so far
#define LOCOMM2              4
#define LOCOMM2Fx            5  //virtual type to store functions
#define LOCOMM3              6  //Not implemented so far
#define LOCOMM3Fx            7  //Not implemented so far
#define LOCOMM4              8  //Not implemented so far
#define LOCOMM4Fx            9  //Not implemented so far
#define LOCOMM5             10  //Not implemented so far
#define LOCOMM5Fx           11  //Not implemented so far
//for  LOCMMXCMD
#define LOCOMMX             12  //alpha version, disable form current versuin
//for LOCNMRACMD and ACCNMRACMD
//NMRA -LOCOXXX must be consequitive numbers different to the types used for MM
#define ACCNMRA              0
#define LOCODCCN14           15
#define LOCODCCN28           16
#define LOCODCCN128          17
#define LOCODCCNfx           18
#define LOCODCCL14           19
#define LOCODCCL28           20
#define LOCODCCL128          21
#define LOCODCCLfx           22
//NULL definition
#define EMPTY                31  //used to init queue

//Type field within the command/type byte
//for PROGCMD
#define NMRA_SERVICE_MODE_DIRECT_write      0
#define NMRA_SERVICE_MODE_DIRECT_verify     1
#define NMRA_SERVICE_MODE_DIRECT_bit        2
#define NMRA_SERVICE_MODE_ADDRESS_write     3
#define NMRA_SERVICE_MODE_ADDRESS_read      4
#define NMRA_SERVICE_MODE_REGISTER_write    5
#define NMRA_SERVICE_MODE_REGISTER_read     6
#define NMRA_SERVICE_MODE_PAGE_write        7
#define NMRA_SERVICE_MODE_PAGE_read         8
#define ACCNMRAwrite                        9
#define ACCNMRAverify                       10
#define ACCNMRAbit                          11
#define LOCODCCNwrite                       12
#define LOCODCNNverify                      13
#define LOCODCNNbit                         14
#define LOCODCCLwrite                       15
#define LOCODCNLverify                      16
#define LOCODCNLbit                         17
#define MM                                  18

#ifdef	__cplusplus
}
#endif

#endif	/* COMMANDS_H */