/*
 * rpic.c
 *
 *  Created on: 06/04/2013
 *      Author: Manolo Serrano
 *      http://tren.enmicasa.net
 *
 *      This file makes used of the WiringPi library developed by Gordon Hendersons (https://projects.drogon.net/raspberry-pi/wiringpi/)
 *      The folder contains the README file and GPL provided by the original distribution, and three of the files needed: wiringPi.c, wiringPi.h and piHiPri.c
 *      The only modification in the source code of such files is a precompiler directive to avoid the compilation when rpic is being compiled for non-arm targets.
 */

#include "rocdigs/impl/rocpi_impl.h"
#include "rocdigs/impl/rocpi/rpic.h"
#ifdef __arm__
#include "rocdigs/impl/rocpi/wiringPi.h"
//#include <wiringPi.h>
//it should be able to work just with this reference... but it doesn't. So I need to include the library and the objects to be used in the rpic folder
//The problem is probably with the way of compiling according to Gordon website:
//"You may also need to add
//
//-I/usr/local/include -L/usr/local/lib -lwiringPi
//
//to the compile line of your program depending on the environment you are using. The important one is -lwiringPi"
//So my solution, so far.. include the library and compile it. With this, it is not necessary to install WiringPi. Only the gpio utility is needed to set up the interruption (check Gordon's web site for more info if needed)
#include <time.h>
static volatile struct timeval start, stop;
#endif


void rocrailready(Boolean ready){
  //discussion on the best approach to identify rpi http://raspberrypi.stackexchange.com/questions/754/how-can-i-detect-that-im-compiling-for-raspberry-pi
  //current approach will compile in any arm device. Not compelte solution
#ifdef __arm__
  if (wiringPiSetup () != -1){
    pinMode (LEDREADY, OUTPUT) ;
    digitalWrite (LEDREADY, ready) ;
    if (ready){
      //set up buttons
      if (wiringPiISR (BUTTONPIN, INT_EDGE_BOTH, &resetbutton) < 0)
        {
          TraceOp.trc(name, TRCLEVEL_INFO, __LINE__, 9999, "Unable to set p the buttons");
        }else{
          TraceOp.trc(name, TRCLEVEL_INFO, __LINE__, 9999, "reset button ready");
        }
    }
  } else{
    TraceOp.trc(name, TRCLEVEL_INFO, __LINE__, 9999, "Not possible to use the RPI GPIO");
  }

#endif
//here an elseif could be included... to send a message to rpic to simulate the behaviour (whenever is possible) of GPIO in rpi

}

void resetbutton (void)
{
#ifdef __arm__
  int readPin;
  int sec;
  readPin=digitalRead (BUTTONPIN);

  if (!readPin){
   // TraceOp.trc(name, TRCLEVEL_INFO, __LINE__, 9999, "Interruption! Pin %i",readPin);
    gettimeofday(&start, NULL);
  }else{
    gettimeofday(&stop, NULL);
    sec=(int)(stop.tv_sec-start.tv_sec);
    //TraceOp.trc(name, TRCLEVEL_INFO, __LINE__, 9999, "Interruption! Pin %i Seconds %i",readPin, sec);
    if (sec>SEC4HALT) {
            TraceOp.trc(name, TRCLEVEL_INFO, __LINE__, 9999, "Halt requested");
            system("sudo halt");
          } else {
            TraceOp.trc(name, TRCLEVEL_INFO, __LINE__, 9999, "Reset requested");
            system("sudo /etc/init.d/rocraild reset");
          }
    //printf("Elapsed %d\n", (int)((stop.tv_sec-start.tv_sec)*1000000ULL+(stop.tv_usec-start.tv_usec)));
  }

#endif

}

unsigned char sendrpiccommand(unsigned char buffer[], unsigned char command, unsigned char type,unsigned char commandlength, iOSerial serialcom){
  unsigned char i;

  buffer[0]=((command<<5)&0b11100000)|type;
    buffer[commandlength]=checksum(buffer, 0, commandlength - 1);

    while (!SerialOp.isUartEmpty(serialcom, True))
      ;
    SerialOp.write(serialcom, buffer, commandlength + 1);
    //TraceOp.trc( name, TRCLEVEL_DEBUG, __LINE__, 9999, "Sendcommand" );

    for (i = 0; i < commandlength + 1; i++) {
      //TraceOp.trc(name, TRCLEVEL_DEBUG, __LINE__, 9999,
      TraceOp.trc(name, TRCLEVEL_INFO, __LINE__, 9999,
          "Command sent buffer %d:  %X", i, buffer[i]);
    }
    i=0;
    SerialOp.read(serialcom, &i, 1);
    //TraceOp.trc(name, TRCLEVEL_DEBUG, __LINE__, 9999, "Response Received %X", i);
    TraceOp.trc(name, TRCLEVEL_INFO, __LINE__, 9999, "Response Received %X", i);

    //error control over i should come here...if error: try to send it again... flush and inform the type of errot (error handler needed)
    //.. if no error...could be the response of a lecture of cv.. and it has to be returned in i.. otherwise i is not used by rocpi

    return i;
}

unsigned char checksum(unsigned char toverify[], unsigned char firstbyte, unsigned char lastbyte) {
    unsigned char i;
    unsigned char sum = toverify[firstbyte];

    for (i = firstbyte; i != lastbyte; i++) {
        sum = sum^toverify[i + 1];
    }
    return sum;
}

